/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.coobra.actions;

import java.awt.event.ActionEvent;

import de.tu_bs.coobra.LocalRepository;
import de.tu_bs.coobra.errors.ErrorHandlerInterface;
import de.tu_bs.coobra.errors.ServerIdentifierChangedException;
import de.uni_paderborn.fujaba.coobra.FujabaChangeManager;
import de.uni_paderborn.fujaba.coobra.FujabaErrorHandler;


/**
 * No comment provided by developer, please add a comment to improve documentation.
 *
 * @author    $Author$
 * @version   $Revision$ $Date$
 */
public class CheckoutAndApplyPatchAction extends CheckoutVersionAction
{
   /**
    *Constructor for class CheckoutAndApplyPatchAction
    */
   public CheckoutAndApplyPatchAction()
   {
      setPatch (true);
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param event  No description provided
    */
   public void actionPerformed (ActionEvent event)
   {
      ErrorHandlerInterface handler = LocalRepository.getErrorHandler();
      LocalRepository.setErrorHandler (
         new FujabaErrorHandler()
         {
            public void handle (RuntimeException exception)
            {
               if (exception instanceof ServerIdentifierChangedException)
               {
                  return;
               }
               super.handle (exception);
            }
         });
      try
      {
         super.actionPerformed (event);
      }
      finally
      {
         LocalRepository.setErrorHandler (handler);
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   protected void createNewProject()
   {
      // do not create a new project
   }



   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   protected LocalRepository createRepository()
   {
      // do not create a new repository
      LocalRepository repository = FujabaChangeManager.getVMRepository();
      repository.setUpdateFilter (getFilter());
      return repository;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   protected CheckoutVersionChooser createVersionChooser()
   {
      CheckoutVersionChooser chooser = new CheckoutVersionChooser();
      chooser.setAllowFreeSelection (true);
      return chooser;
   }
}

/*
 * $Log$
 * Revision 1.1.2.1  2006/03/07 17:49:59  l3_g5
 * added coobra action "apply as patch"
 *
 */
